# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging
import six
from six import BytesIO
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String

logger = logging.getLogger("driverng.cfx_biorad")


class CfxBioradBase(Splitted):

    SEPARATOR = ";"


class CfxBioradResult(object):
    FIELDS = [
        String("well"),
        String("fluor"),
        String("target"),
        String("content"),
        String("sample"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class CfxBioradResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: CfxBioradResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode("utf-8"))
        else:
            buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        self.end_date = None
        self.file_name = None
        self.operator_user = None
        self.instrument_serial_number = None
        self.notes = None
        for row in reader:
            if six.PY2:
                line = [
                    c.decode("utf-8") for c in row
                ]  # PY2 csv reader est incompatible avec unicode
            else:
                line = row
            if not len(line):
                continue
            if line[0] == "Run Ended":
                try:
                    self.end_date = datetime.datetime.strptime(
                        line[1].strip()[:19], "%m/%d/%Y %H:%M:%S"
                    )
                except ValueError as e:
                    logger.info("La date des résultat n'est pas au bon format %s", e)
            if line[0] == "File Name":
                self.file_name = line[1].strip()
            if line[0] == "Created By User":
                self.operator_user = line[1].strip()
            if line[0] == "Base Serial Number":
                self.instrument_serial_number = line[1].strip()
            if line[0] == "Notes":
                self.notes = line[1].strip()
            if len(line) >= 6:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "cfx_biorad", "label": "Cfx Bio-Rad CSV"}

    OPTIONS = [
        MessageV2.Option("analyseur_traites", label="Liste des analyseurs traités"),
        MessageV2.BooleanOption(
            "extra_results", label="Récupérer plusieurs résultats", default=False
        ),
    ]

    def __init__(self, **kwargs):
        analyseur_traites = kwargs.pop("analyseur_traites", None)
        self.analyseur_traites = (
            analyseur_traites.split(",") if analyseur_traites is not None else None
        )
        self.extra_results = kwargs.pop("extra_results", False)
        super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        obj = CfxBioradResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if not m.target and not m.sample:
                continue
            elif m.sample == "Sample" and m.well == "Well":
                RESULTS_CODES = m.results
                continue
            if (
                self.analyseur_traites is not None
                and message.instrument_serial_number not in self.analyseur_traites
            ):
                logger.error(
                    "L'instrument %s n'est pas traité par la connexion"
                    % message.instrument_serial_number
                )
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(
                    sample_id=m.sample, rack="CFX-{}".format(m.well), type="patient"
                )
                if message.file_name:
                    sample.results.get_or_create(
                        code="File Name", value=message.file_name[:-5]
                    )
                if message.notes:
                    sample.results.get_or_create(code="Notes", value=message.notes)
                if message.operator_user:
                    sample.results.get_or_create(
                        code="Created By User", value=message.operator_user
                    )
                if m.well:
                    sample.results.get_or_create(code="RACK", value=m.well)
                for index, result in enumerate(m.results):
                    if RESULTS_CODES[index] and result:
                        # Conserver la compatibilité sur l'ancien format avec
                        # présence d'uniquement un seul résultat dans la colonne Cq
                        if not self.extra_results:
                            sample.results.add(
                                code="{}{}".format(m.fluor, m.target),
                                value=result,
                                instrument=message.instrument_serial_number,
                                date=message.end_date,
                                operator=message.operator_user,
                                comments=[message.notes],
                            )
                        else:
                            sample.results.add(
                                code="{}{}{}".format(
                                    m.fluor, m.target, RESULTS_CODES[index].strip()
                                ),
                                value=result,
                                instrument=message.instrument_serial_number,
                                date=message.end_date,
                                operator=message.operator_user,
                                comments=[message.notes],
                            )
        return worklist
