# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)


class IsedSegmentH(HeaderSegment):
    manufacturer_id = HprimString((5, 1))
    product_name_id = HprimString((5, 2))
    software_version = HprimString((5, 3))
    analyzer_id = HprimString((5, 4))
    procesing_id = HprimString(12)
    version = HprimString(13)
    date = HprimDate(14)

    patients = HprimForeign("P", "patients")
    end = HprimForeign("L", "header", one2one=True, required=True)


class IsedSegmentP(Segment):
    laboratory_assigned_patient_id = HprimString(4)

    orders = HprimForeign("O", "orders")


class IsedSegmentO(Segment):
    sample_id = HprimString((3, 1))
    rotor_location = HprimString((3, 2))
    universal_test_id = HprimString((5, 4))
    report_types = HprimString(26)

    results = HprimForeign("R", "results")


class IsedSegmentR(Segment):
    universal_test_id = HprimString((3, 4))
    loinc_code = HprimString((3, 5))
    value = HprimString(4)
    unit = HprimString(5)
    abnormal_flag = HprimString(7)
    result_status = HprimString(9)
    date_test_starting = HprimDate(12)
    date_test_completed = HprimDate(13)
    instrument = HprimString(14)


class IsedSegmentL(Segment):
    terminaison_code = HprimString(3)


class Driver(AstmMessageV2):

    SPEC = {"code": "ised", "label": "ISED ELITECH ASTM"}
    SEGMENT_CLASS_MAP = {
        "H": IsedSegmentH,
        "P": IsedSegmentP,
        "O": IsedSegmentO,
        "R": IsedSegmentR,
        "L": IsedSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"
    OPTIONS = [
        AstmMessageV2.Option(
            "ised_qc_prefix", label="Préfixe QC au début de la référence", default=""
        )
    ]

    def __init__(self, **kwargs):
        self.qc_prefix = kwargs.pop("ised_qc_prefix", "")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = IsedSegmentH(type_segment="H", **kwargs)
        message.end = IsedSegmentL(type_segment="L")
        return message

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if self.qc_prefix and seg_o.sample_id.startswith(self.qc_prefix):
                    sample = worklist.get_or_create(
                        control_lot=seg_o.sample_id,
                        sample_id=seg_o.sample_id,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.sample_id,
                        rack=seg_o.rotor_location,
                        type="patient",
                    )
                for seg_r in seg_o.results:
                    sample.results.add(
                        code=seg_r.universal_test_id,
                        value=seg_r.value,
                        flags=[seg_r.abnormal_flag],
                        unit=seg_r.unit,
                        date=seg_r.date_test_completed,
                        instrument=seg_r.instrument,
                    )
        return worklist
