# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.exceptions import ValidationError
from driverng import Message
from driverng.splitted import Date, Splitted, String


def datetime_combine(d1, d2):
    if d1 is not None and d2 is not None:
        d1 = d1.replace(hour=d2.hour, minute=d2.minute)
    return d1


class SirContainer(list):
    def __init__(self, separator="|"):
        self.separator = separator
        super(SirContainer, self).__init__()

    def add(self, *args, **kwargs):
        obj = Splitted(fields=self.fields, separator=self.separator, *args, **kwargs)
        self.append(obj)
        return obj

    def to_python(self, data):
        obj = self.add()
        obj.parse(data)
        return obj

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class SirBase(Splitted):
    SEPARATOR = "|"


class SirDemande(SirBase):
    FIELDS = [
        String("type", default="D"),
        String("indicateur"),
        # U urgent ,T tél
        String("urgent"),
        String("sample_id"),
        String("nom"),
        String("prenom"),
        # M, F
        String("sexe"),
        Date("date_naissance", fmt_date="%Y%m%d"),
        Date("date_entree", fmt_date="%Y%m%d"),
        String("no_ipp"),
        String("no_hospitalise"),
        # O/N
        String("femme_enceinte"),
        String("temperature"),
        String("nom_jf"),
        String("code_postal"),
        String("comment"),
        String("reserve17"),
        String("no_insee"),
        String("validation"),
        Date("date_validation", fmt_date="%Y%m%d"),
        Date("heure_validation", fmt_date="%H:%M"),
        String("valideur"),
        String("service"),
        # no chambre ou medecin 2
        String("medecin1"),
        String("medecin2"),
        String("no_chambre"),
        String("no_lit"),
        Date("date_prel", fmt_date="%Y%m%d"),
        Date("heure_prel", fmt_date="%H%M"),
        Date("date_reception", fmt_date="%Y%m%d"),
        Date("heure_reception", fmt_date="%H%M"),
        String("prelevement"),
        String("prel_origine"),
        String("prel_localisation"),
        String("prel_libre"),
        Date("date_envoie", fmt_date="%Y%m%d"),
        Date("heure_envoie", fmt_date="%H%M"),
        String("no_analyse"),
        String("no_labo"),
        String("no_zone_labo"),
        String("nom_galerie"),
        # no d'ordre du germe
        String("no_isolat"),
        String("germe_identifie"),
        String("germe_complement"),
        String("nom_antibiogramme"),
        String("no_germe"),
        String("reserve47"),
        String("reserve48"),
        String("reserve49"),
        String("reserve50"),
        String("quantite_germes"),
        String("profil_biochimique"),
        String("serotype"),
        String("no_souchier"),
        String("validation_isolat"),
        Date("date_validation_isolat", fmt_date="%Y%m%d"),
        Date("heure_validation_isolat", fmt_date="%H:%M"),
        String("valideur_isolat"),
        String("facteur_dilution"),
        String("support_prel"),
    ]

    def __init__(self, *args, **kwargs):
        self._locked = False
        self.informations = []
        self.resultats = []
        super(SirDemande, self).__init__(*args, **kwargs)

    def to_string(self):
        lines = (
            [super(SirDemande, self).to_string()]
            + [e.to_string() for e in self.informations]
            + [e.to_string() for e in self.resultats]
        )
        return "\r\n".join(lines)


class SirUrl(SirBase):
    FIELDS = [
        String("type", default="U"),
        String("reserve1"),
        String("reserve2"),
        String("reserve3"),
        String("type_document"),
        String("reserve4"),
        String("reserve5"),
        String("reserve6"),
        String("extension_fichier"),
        String("url"),
    ] + [String("reserve%s" % i) for i in range(11, 21)]


class SirInformation(SirBase):
    FIELDS = [
        String("type", default="I"),
        String("reserve1"),
        String("reserve2"),
        String("reserve3"),
        String("code"),
        String("reserve4"),
        String("reserve5"),
        String("reserve6"),
        String("reserve7"),
        String("valeur"),
    ] + [String("reserve%s" % i) for i in range(11, 21)]


class SirResultatDemande(SirBase):
    FIELDS = [
        String("type", default="I"),
        String("reserve1"),
        String("test"),
        String("reserve2"),
        String("reserve3"),
        String("resultat"),
        Date("date_resulat", fmt_date="%Y%m%d"),
        Date("heure_resulat", fmt_date="%H:%M"),
        String("automate"),
        String("operateur"),
        String("reserve4"),
        String("reserve5"),
        String("reserve6"),
        String("indicateur_masque"),
        String("commentaire"),
    ] + [String("reserve%s" % i) for i in range(16, 36)]


class SirPatient(SirBase):
    FIELDS = [
        String("type", default="P"),
        # P 1er envoie , R reémission
        String("reemission", default="P"),
    ] + [String("reserve%s" % i) for i in range(3, 8)]
    FIELDS += [
        String("demandeur1"),
        String("demandeur2"),
        String("demandeur3"),
        String("prel_type"),
        String("prel_origine"),
        String("prel_libre"),
        String("germe_identifie"),
        String("germe_complement"),
        String("qte_germe"),
        String("sample_id"),
        String("no_analyse"),
        Date("date_hospitalisation", fmt_date="%Y%m%d"),
        String("nom_antibiogramme"),
        String("no_sir"),
        Date("date_naissance", fmt_date="%Y%m%d"),
        String("femme_enceinte"),
        Date("heure_prel", fmt_date="%H:%M"),
        String("nosocomiale"),
        String("serotype"),
        String("no_souchier"),
        String("no_isolat"),
        String("prelevement_localisation"),
        String("temperature"),
        Date("date_reception", fmt_date="%Y%m%d"),
        Date("heure_reception", fmt_date="%H:%M"),
        String("code_postal"),
        String("niveau_validation"),
        String("no_labo"),
        String("origine_dossier"),
        String("profil_biochimique"),
        String("type_dossier"),
        String("resultat_qualitatif"),
        String("reserve40"),
        String("germe_id"),
        Date("date_validation", fmt_date="%Y%m%d"),
        Date("heure_validation", fmt_date="%H:%M"),
        Date("date_test", fmt_date="%Y%m%d"),
        Date("heure_test", fmt_date="%H:%M"),
        String("operateur"),
    ]
    FIELDS += [String("reserve%s" % i) for i in range(48, 61)]
    FIELDS.append(String("score_bruker"))

    def __init__(self, *arg, **kwargs):
        self._locked = False
        for attr in [
            "informations",
            "resultats",
            "ensemencements",
            "antibiotiques",
            "commentaires",
            "tests",
        ]:
            setattr(self, attr, [])
        super(SirBase, self).__init__(*arg, **kwargs)


class SirEnsemence(SirBase):
    FIELDS = [String("type", default="S")]
    FIELDS += [String("reserve%s" % i) for i in range(2, 5)]
    FIELDS += [
        String("no_support"),
        String("nom_support"),
        String("no_lot_support"),
        Date("date_peremption_support", fmt_date="%Y%m%d"),
        Date("heure_peremption_support", fmt_date="%H%M"),
        String("nom_milieu"),
        String("no_lot_milieu"),
        Date("date_peremption_milieu", fmt_date="%Y%m%d"),
        Date("heure_peremption_milieu", fmt_date="%H%M"),
        String("no_depot"),
        String("nom_outil_depot"),
        String("no_lot_outil"),
        Date("date_peremption_outil", fmt_date="%Y%m%d"),
        Date("heure_peremption_outil", fmt_date="%H%M"),
        String("nom_ensemencement"),
        String("nom_outil_ensemencement"),
        String("no_lot_outil_ensemencement"),
        Date("date_peremption_outil_ensemencement", fmt_date="%Y%m%d"),
        Date("heure_peremption_outil_ensemencement", fmt_date="%H%M"),
        Date("date_ensemencement", fmt_date="%Y%m%d"),
        Date("heure_ensemencement", fmt_date="%H%M"),
        String("operateur"),
        String("no_automate"),
    ]
    FIELDS += [String("reserve%s" % i) for i in range(28, 46)]


class SirResultat(SirBase):
    FIELDS = [
        String("type", default="R"),
        String("reserve2"),
        String("code_test"),
        String("resultat_final"),
        String("resultat_code"),
        String("resultat_quantite"),
        String("reserve7"),
        String("id_automate"),
        Date("date_test", fmt_date="%Y%m%d"),
        Date("heure_test", fmt_date="%H:%M"),
        String("operateur"),
        String("reserve12"),
        String("type_rubrique"),
        String("commentaire_rubrique"),
        String("indic_masque"),
    ]
    FIELDS += [String("reserve%s" % i) for i in range(16, 37)]


class SirAntibiotique(SirBase):
    FIELDS = [
        String("type", default="A"),
        String("reserve2"),
        String("nom_antibiotique"),
        String("diametre"),
        String("cmi"),
        String("valeur_brute"),
        String("valeur_interprete"),
        String("resultat_complementaire"),
        String("reserve9"),
        String("posologie"),
        String("voie_administration"),
        String("cout"),
        String("remarque"),
        String("remarque_complementaire"),
        String("indic_masque"),
        String("reserve16"),
        String("reserve17"),
        String("diametre_bas"),
        String("diametre_haut"),
        String("cmi_bas"),
        String("cmi_haut"),
        String("origine"),
        Date("date_test", fmt_date="%Y%m%d"),
        Date("heure_test", fmt_date="%H:%M"),
        String("operateur"),
        String("coef_cmi"),
        String("quotient_inhibiteur"),
        String("auic"),
        String("code_barre_antibiotique"),
        Date("date_peremption", fmt_date="%Y%m%d"),
        Date("heure_peremption", fmt_date="%H:%M"),
        String("no_lot"),
    ]
    FIELDS += [String("reserve%s" % i) for i in range(31, 36)]


class SirCommentaire(SirBase):
    FIELDS = [String("type", default="C"), String("reserve2"), String("commentaire")]
    FIELDS += [String("reserve%s" % i) for i in range(3, 8)]


class SirTest(SirBase):
    FIELDS = [
        String("type", default="T"),
        String("reserve2"),
        String("nom_test"),
        String("reserve4"),
        String("valeur_test"),
    ]
    FIELDS += [String("reserve%s" % i) for i in range(6, 13)]


class SirMultipleMessage(list):
    MESSAGE_TYPE_MAP = {
        "D": SirDemande,
        "I": SirInformation,
        "P": SirPatient,
        "R": SirResultat,
        "A": SirAntibiotique,
        "C": SirCommentaire,
        "S": SirEnsemence,
        "T": SirTest,
    }

    def add(self, message_type="None", **kwargs):
        obj = self.MESSAGE_TYPE_MAP[message_type](message_type=message_type, **kwargs)
        self.append(obj)
        return obj

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(Message):
    SPEC = {"code": "sir", "label": "SIR WEB I2A"}
    MESSAGE_TYPE_MAP = {
        "D": SirDemande,
        "I": SirInformation,
        "P": SirPatient,
        "R": SirResultat,
        "A": SirAntibiotique,
        "C": SirCommentaire,
        "S": SirEnsemence,
        "T": SirTest,
    }

    OPTIONS = [
        Message.BooleanOption(
            "sir_no_information",
            label="Ne pas transmettre le segment Information avec la demande",
            default=False,
        ),
        Message.Option("sir_default_cmi", label="CMI par défaut", default="-"),
        Message.BooleanOption(
            "sir_ident_prel",
            label="Récupération de l'identification du germe dans le résultat de l'ATB dans le code réponse automate IDENT_<PREL>",
            default=False,
        ),
        Message.BooleanOption(
            "sir_identification_atb",
            label="Mettre le germe de l'ATB dans une réponse automate IDENT_",
            default=False,
        ),
        Message.BooleanOption(
            "sir_identification_spectro",
            label="Mettre le score et l'incide de confiance dans un trame SPECTRO",
            default=True,
        ),
        Message.Option(
            "sir_prefixe_germe",
            label="Préfixe des textes codés des germes (par défaut G)",
            default="G",
        ),
        Message.Option(
            "sir_prefixe_antibio",
            label="Préfixe des textes codés des antibiotiques (par défaut A)",
            default="A",
        ),
        Message.BooleanOption(
            "sir_no_labo_field",
            label="Transmission du code labo dans le champ D39",
            default=False,
        ),
        Message.BooleanOption(
            "sir_get_diametre_cutoff",
            label="Traitement du diametre et du cutoff dans les résultats des ATB",
            default=False,
        ),
        Message.Option(
            "sir_prefixe_analyseur",
            label="Indiquer le préfixe à mettre devant le code analyseur du champ P36",
            default="",
        ),
    ]

    def __init__(self, **kwargs):
        self.sir_no_information = kwargs.pop("sir_no_information", False)
        self.sir_default_cmi = kwargs.pop("sir_default_cmi", "-")
        self.sir_ident_prel = kwargs.pop("sir_ident_prel", False)
        self.sir_identification_atb = kwargs.pop("sir_identification_atb", False)
        self.sir_prefixe_germe = kwargs.pop("sir_prefixe_germe", "G")
        self.sir_prefixe_antibio = kwargs.pop("sir_prefixe_antibio", "A")
        self.sir_no_labo_field = kwargs.pop("sir_no_labo_field", False)
        self.sir_get_diametre_cutoff = kwargs.pop("sir_get_diametre_cutoff", False)
        self.sir_identification_spectro = kwargs.pop("sir_identification_spectro", True)
        self.sir_prefixe_analyseur = kwargs.pop("sir_prefixe_analyseur", "")
        return super(Driver, self).__init__(**kwargs)

    def create(self):
        return SirMultipleMessage()

    def get_message_type(self, message):
        return self.ORDER

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        # un fichier peut contenir plusieurs patients
        lines = (li for li in data.splitlines() if li)

        sirMsg = SirContainer()
        patient = None
        for line in lines:
            message_type = line.split("|", 1)[0]
            if patient is None:
                if message_type != "P":
                    raise ValidationError("Segment '%s' sans patient" % (message_type,))
            obj = self.MESSAGE_TYPE_MAP[message_type]()
            obj.parse(line)
            if message_type in ("P", "D"):
                patient = obj
                sirMsg.append(patient)
            else:
                if message_type == "I":
                    patient.informations.append(obj)
                elif message_type == "R":
                    patient.resultats.append(obj)
                elif message_type == "S":
                    patient.ensemencements.append(obj)
                elif message_type == "T":
                    patient.tests.append(obj)
                elif message_type == "A":
                    patient.antibiotiques.append(obj)
                elif message_type == "C":
                    patient.commentaires.append(obj)
                else:
                    raise ValidationError("Segment '%s' no mapped" % (message_type,))
        return sirMsg

    def get_order_message(self, worklist, context):
        """Pour encoder des résultats bruker, en prenant une worklist afin de
        renvoyer des trames, utiliser la fonction `encode_worklist`.
        """
        if len(worklist.samples) == 0:
            return None
        if context["query"] is not None:
            raise NotImplementedError("context %s" % (context,))
        samples = worklist.get_worklist(
            unique_together=["action", "priority"],
            action={"del": "S", worklist.ELSE: ""},
            priority={"A": "U", "S": "U", worklist.ELSE: ""},
        )
        msg = SirContainer()
        for sample, action, priority in samples:
            origine = ""
            localisation = ""
            for order in sample.orders:
                list_id_germe = []
                for result in sample["results"]:
                    if result["code"] and result["code"].startswith("ORIG"):
                        origine = result["value"]
                    elif result["code"] and result["code"].startswith("LOCAL"):
                        localisation = result["value"]
                    elif "germes" in result["resu_json"]:
                        for germe in result["resu_json"]["germes"]:
                            code_germe = ""
                            if "Id" in germe:
                                code_germe = germe.get("Id")
                            elif "id" in germe:
                                code_germe = germe.get("id")
                            elif "coli" in germe:
                                code_germe = germe.get("coli")
                            if code_germe:
                                list_id_germe.append(
                                    code_germe[2:]
                                    if code_germe.startswith("G_")
                                    else code_germe
                                )

                if (
                    sample.physiologie is not None
                    and "GROSSESSE" in sample.physiologie.upper()
                ):
                    indicateur_grossesse = "O"
                else:
                    indicateur_grossesse = "N"
                order_code = ""
                if order.code.startswith("ORIG_"):
                    order_code = order.code[5:]
                elif order.code.startswith("LOCAL_"):
                    order_code = order.code[6:]
                else:
                    order_code = order.code
                patient = self.create()
                if list_id_germe:
                    for index, germe in enumerate(list_id_germe):
                        patient.add(
                            "D",
                            indicateur=action,
                            urgent=priority,
                            sample_id=sample.sample_id,
                            demandeur2=sample.service,
                            demandeur3=sample.location,
                            no_ipp=sample.lab_patient_id,
                            no_hospitalise=sample.alternate_patient_id,
                            date_entree=sample.admission_date,
                            no_lit=sample.bed,
                            nom=sample.last_name,
                            prenom=sample.first_name,
                            date_naissance=sample.birthdate,
                            sexe=sample.sex,
                            date_prel=sample.collection_date,
                            heure_prel=sample.collection_date,
                            date_reception=sample.request_date,
                            heure_reception=sample.request_date,
                            prelevement=order_code,
                            prel_origine=origine or sample.specimen,
                            prel_localisation=localisation,
                            nom_jf=sample.maiden_name,
                            code_postal=sample.postal_code,
                            femme_enceinte=indicateur_grossesse,
                            no_analyse=sample.lab_patient_id,
                            no_labo=sample.lab if self.sir_no_labo_field else "",
                            no_zone_labo=sample.lab
                            if not self.sir_no_labo_field
                            else "",
                            service=sample.service,
                            medecin1=sample.physician,
                            medecin2=sample.medecin2,
                            no_isolat=index + 1,
                            germe_identifie=germe,
                        )
                else:
                    patient = self.create()
                    patient.add(
                        "D",
                        indicateur=action,
                        urgent=priority,
                        sample_id=sample.sample_id,
                        demandeur2=sample.service,
                        demandeur3=sample.location,
                        no_ipp=sample.lab_patient_id,
                        no_hospitalise=sample.alternate_patient_id,
                        date_entree=sample.admission_date,
                        no_lit=sample.bed,
                        nom=sample.last_name,
                        prenom=sample.first_name,
                        date_naissance=sample.birthdate,
                        sexe=sample.sex,
                        date_prel=sample.collection_date,
                        heure_prel=sample.collection_date,
                        date_reception=sample.request_date,
                        heure_reception=sample.request_date,
                        prelevement=order_code,
                        prel_origine=origine or sample.specimen,
                        prel_localisation=localisation,
                        nom_jf=sample.maiden_name,
                        code_postal=sample.postal_code,
                        femme_enceinte=indicateur_grossesse,
                        no_analyse=sample.lab_patient_id,
                        no_labo=sample.lab if self.sir_no_labo_field else "",
                        no_zone_labo=sample.lab if not self.sir_no_labo_field else "",
                        service=sample.service,
                        medecin1=sample.physician,
                        medecin2=sample.medecin2,
                    )
                if action != "S":
                    for result in sample["results"]:
                        if result["code"].startswith("ORIG_"):
                            result_code = result["code"][5:]
                        elif result["code"].startswith("LOCAL_"):
                            result_code = result["code"][6:]
                        else:
                            result_code = result["code"]
                        if not self.sir_no_information:
                            patient[0].informations.append(
                                SirInformation(code=result_code, valeur=result["value"])
                            )
                msg.append(patient)

        if len(msg):
            return msg

    def get_result_dict(self, message):
        result_dict = {}
        for patient in message:
            resus_dict = result_dict.setdefault(
                patient.sample_id, {"rack": None, "resultats": []}
            )
            for result in patient.resultats:
                date = datetime_combine(result.date_test, result.heure_test)
                comments = []
                if result.commentaire_rubrique:
                    comments.append(result.commentaire_rubrique)
                if result.type_rubrique:
                    comments.append(result.type_rubrique)
                resus_dict["resultats"].extend(
                    [
                        {
                            "test": result.code_test,
                            "resu": result.resultat_final,
                            "date": date,
                            "analyseur": result.id_automate,
                            "commentaires": comments,
                        },
                        {
                            "test": result.code_test + "_CODE",
                            "resu": result.resultat_code,
                            "date": date,
                            "analyseur": result.id_automate,
                            "commentaires": comments,
                        },
                        {
                            "test": result.code_test + "_QTE",
                            "resu": result.resultat_quantite,
                            "date": date,
                            "analyseur": result.id_automate,
                            "commentaires": comments,
                        },
                    ]
                )

            if (
                patient.germe_identifie is not None
                and patient.germe_identifie.upper() != "CYTOLOGIE"
            ):
                if (
                    self.sir_identification_atb
                    and patient.germe_identifie
                    and patient.antibiotiques
                ):
                    # Identification des germes même s'il y a des antibiotiques
                    # selon activation de l'option sur la connexion
                    germe_ident_atb = {
                        "test": patient.prel_type,
                        "resu": "G_%s" % (patient.germe_identifie.upper()),
                        "date": datetime_combine(
                            patient.date_validation, patient.heure_validation
                        ),
                        "seq": patient.no_isolat,
                        "analyseur": "{}{}".format(
                            self.sir_prefixe_analyseur, patient.origine_dossier
                        ),
                        "nom_antibiogramme": patient.nom_antibiogramme,
                        "atb": [],
                        "commentaires": [],
                    }
                    germe_ident_atb["test"] = "IDENT_" + germe_ident_atb["test"]
                    germe_ident_atb["resu"] = {
                        "type_resultat": "identification_germes",
                        "germes": [],
                    }
                    germe_ident_atb["seq"] = "0"
                    if germe_ident_atb["test"] in resus_dict:
                        germe_ident_atb["resu"]["germes"] = result_dict[
                            germe_ident_atb["test"]
                        ]["resu"].get("germes", [])
                    germe_ident_atb["resu"]["germes"].append(
                        {
                            "id": "G_%s" % (patient.germe_identifie,),
                            "rang": int(patient.no_isolat),
                        }
                    )
                    resus_dict["resultats"].append(germe_ident_atb)

                germe = {
                    "test": patient.prel_type,
                    "resu": "%s_%s"
                    % (self.sir_prefixe_germe, patient.germe_identifie.upper()),
                    "date": datetime_combine(
                        patient.date_validation, patient.heure_validation
                    ),
                    "seq": patient.no_isolat,
                    "analyseur": "{}{}".format(
                        self.sir_prefixe_analyseur, patient.origine_dossier
                    ),
                    "nom_antibiogramme": patient.nom_antibiogramme,
                    "atb": [],
                    "commentaires": [],
                }
                if patient.germe_identifie and not patient.antibiotiques:
                    # Identification des germes
                    # Version JSON
                    germe["test"] = "IDENT_" + germe["test"]
                    germe["resu"] = {
                        "type_resultat": "identification_germes",
                        "germes": [],
                    }
                    germe["seq"] = "0"
                    if germe["test"] in resus_dict:
                        germe["resu"]["germes"] = result_dict[germe["test"]][
                            "resu"
                        ].get("germes", [])
                    result_ident = {
                        "id": "{}_{}".format(
                            self.sir_prefixe_germe, patient.germe_identifie
                        ),
                        "rang": int(patient.no_isolat),
                    }
                    if patient.score_bruker:
                        result_ident["score"] = patient.score_bruker
                    germe["resu"]["germes"].append(result_ident)
                    resus_dict["resultats"].append(germe)

                    # Version classique avec réponse automate
                    resus_dict["resultats"].append(
                        {
                            "test": "GERME_{}_{}".format(
                                patient.no_isolat, patient.prel_type
                            ),
                            "resu": "G_{}".format(patient.germe_identifie),
                        }
                    )
                elif patient.antibiotiques:
                    # Antibiogramme
                    for atb in patient.antibiotiques:
                        atb_result = (
                            (
                                "ANTIBIO",
                                "%s_%s"
                                % (
                                    self.sir_prefixe_antibio,
                                    atb.nom_antibiotique.upper(),
                                ),
                            ),
                            ("CMI", atb.cmi or self.sir_default_cmi),
                            ("SENSI", atb.valeur_interprete),
                        )
                        if self.sir_get_diametre_cutoff:
                            atb_result += (("DIAMETRE", atb.diametre),)
                            if atb.cmi_bas and atb.cmi_haut:
                                atb_result += (
                                    (
                                        "CUTOFF",
                                        "{}-{}".format(atb.cmi_bas, atb.cmi_haut),
                                    ),
                                )
                        germe["atb"].append(atb_result)
                    if germe["atb"]:
                        resus_dict["resultats"].append(germe)
                        resus_dict["resultats"].append(
                            {"test": "NOSIR", "resu": patient.no_sir}
                        )
                    comment_atb = []
                    # traitement des commantaires
                    for i, c in enumerate(patient.commentaires):
                        comment_atb.append(c.commentaire)
                    if len(comment_atb):
                        comment = "\n".join(comment_atb)
                        germe["commentaires"] = [comment]
                    if self.sir_ident_prel:
                        germe_identifie = "G_{}".format(patient.germe_identifie.upper())
                        resus_dict["resultats"].append(
                            {
                                "test": "IDENT_JSON_{}".format(patient.prel_type),
                                "resu": {
                                    "type_resultat": "identification_germes",
                                    "germes": [
                                        {
                                            "id": germe_identifie,
                                            "rang": int(patient.no_isolat),
                                        }
                                    ],
                                },
                                "seq": "0",
                            }
                        )

            for i, t in enumerate(patient.tests):
                resus_dict["resultats"].append(
                    {"test": t.nom_test, "resu": t.valeur_test}
                )
            for i, e in enumerate(patient.ensemencements):
                if e.no_support:
                    dl = datetime_combine(
                        e.date_peremption_support, e.heure_peremption_support
                    )
                    if dl is not None:
                        date_lot = "péremption le %s" % (dl.strftime("%d/%m/%Y %H:%M"),)
                    else:
                        date_lot = ""
                    resus_dict["resultats"].append(
                        {
                            "test": "NO_SUPPORT",
                            "resu": e.no_support,
                            "commentaires": [
                                "%s lot no: %s %s"
                                % (e.nom_support, e.no_lot_support, date_lot)
                            ],
                        }
                    )
                if e.nom_milieu:
                    dl = datetime_combine(
                        e.date_peremption_milieu, e.heure_peremption_milieu
                    )
                    if dl is not None:
                        date_lot = "péremption le %s" % (dl.strftime("%d/%m/%Y %H:%M"),)
                    else:
                        date_lot = ""
                    resus_dict["resultats"].append(
                        {
                            "test": "NOM_MILIEU",
                            "resu": e.nom_milieu,
                            "commentaires": [
                                "lot no: %s %s" % (e.no_lot_milieu, date_lot)
                            ],
                        }
                    )
                if e.no_depot:
                    dl = datetime_combine(
                        e.date_peremption_outil, e.heure_peremption_outil
                    )
                    if dl is not None:
                        date_lot = "péremption le %s" % (dl.strftime("%d/%m/%Y %H:%M"),)
                    else:
                        date_lot = ""
                    resus_dict["resultats"].append(
                        {
                            "test": "NO_DEPOT",
                            "resu": e.no_depot,
                            "commentaires": [
                                "%s lot no: %s %s"
                                % (e.nom_outil_depot, e.no_lot_outil, date_lot)
                            ],
                        }
                    )
                # Nom de l'ensemencement
                if e.nom_ensemencement:
                    dl = datetime_combine(
                        e.date_peremption_outil_ensemencement,
                        e.heure_peremption_outil_ensemencement,
                    )
                    if dl is not None:
                        date_lot = "péremption le %s" % (dl.strftime("%d/%m/%Y %H:%M"),)
                    else:
                        date_lot = ""
                    resus_dict["resultats"].append(
                        {
                            "test": "ENSEMENCEMENT",
                            "resu": e.nom_ensemencement,
                            "date": datetime_combine(
                                e.date_ensemencement, e.heure_ensemencement
                            ),
                            "commentaires": [
                                "%s lot no: %s %s"
                                % (
                                    e.nom_outil_ensemencement,
                                    e.no_lot_outil_ensemencement,
                                    date_lot,
                                )
                            ],
                        }
                    )

        return result_dict

    def encode_worklist(self, worklist, context):
        """Encodage d'une worklist de contexte "result"
        TODO: En passant en ASTMV2, intégrer le code de get_order_message,
        et optionner ce code en créant une option BACMS RESULTAT
        """
        if worklist.context == "result":
            msg = SirContainer()
            for sample in worklist.samples:
                patient = self.create()
                for result in sample.results:
                    patient.add(
                        "P",
                        sample_id=sample.sample_id,
                        germe_identifie=result.value,
                        no_isolat=result.sub_code,
                        date_validation=result.date,
                        heure_validation=result.date,
                        prel_type=result.code,
                    )
                    if (
                        self.sir_identification_spectro
                        and result.code.upper() == "SPECTRO"
                    ):
                        patient[-1].score_bruker = result.status
                msg.append(patient)
            if len(msg):
                return msg
        else:
            return super(Driver, self).encode_worklist(worklist, context)
